class Lang:
    type_error = "参数 {target} 的类型不正确"
    content_error = "参数 {target} 不正确"
    sequence_form_error = "不正确的序列形式: {target}"
    validate_reject = "{target} 校验失败"
    pattern_head_or_tail_error = "不允许正则表达式 {target} 头尾部分使用 '^' 或 '$' "


lang = Lang()
